#PYTHON module for ploting 

import numpy as np
import matplotlib.pyplot as plt
import re

import yaiv.utils as ut

# PLOTTING BANDS----------------------------------------------------------------

def __insert_space_before_minus(string):
    """A simple tool to reformat strings and improve reading"""
    new_string=''
    for i in string:
        if i=='-':
            new_string=new_string+' -'
        else:
            new_string=new_string+i
    return new_string

def __ticks_generator(vectors,ticks,grid=None):
    """From the real vectors, the High Sym points for the path in crystal reciprocal space units and the 
    grid, it generates the positions for the hight sym points (tick_pos) and grid (grid_pos):
    
    vectors=[[a1,a2,a3],...,[c1,c2,c3]]
    ticks=[[tick1x,tick1y,tick1z,100],...,[ticknx,tickny,ticknz,1]]
    grid=[[grid1x,grid1y,grid1z],...,[gridnx,gridny,gridnz]]
    
    returns either tick_pos or tick_pos, grid_pos
    """
    K_vec=ut.K_basis(vectors)
    path=0
    tick0=ticks[0][:3]
    ticks_pos=np.array(0)
    for i in range(1,ticks.shape[0]):
        tick1=ticks[i][:3]
        if  ticks[i-1][3]==1:
            tick0=tick1
        else:
            if np.any(grid!= None):
                for point in grid:
                    dist=__lineseg_dist(point,tick0,tick1)
                    if np.around(dist,decimals=3)==0:
                        if np.around(np.linalg.norm(point-tick0),decimals=3)==0:
                            delta=0
                        elif np.around(np.linalg.norm(point-tick1),decimals=3)==0:
                            vector=(tick1-tick0)
                            delta=np.linalg.norm(ut.cryst2cartesian(vector,K_vec))
                        else:
                            vector=(point-tick0)
                            delta=np.linalg.norm(ut.cryst2cartesian(vector,K_vec))
                        try:
                            if np.all(grid_ticks!=(path+delta)):  #grid_ticks are not degenerate
                                grid_ticks=np.append(grid_ticks,path+delta)
                        except NameError:
                            grid_ticks=np.array(path+delta)
            vector=(tick1-tick0)
            delta=ut.cryst2cartesian(vector,K_vec)
            path=path+np.linalg.norm(delta)
            ticks_pos=np.append(ticks_pos,path)
            tick0=tick1
    if np.any(grid!= None):
        return ticks_pos, grid_ticks
    else:
        return ticks_pos

def __process_electron_bands(filename,filetype=None,vectors=np.array(None)):
    """Process the bands from various file types with each band separately separated by blank lines
    to a matrix where each column is a band and first column is x axis
    
    filename = File with the bands
    filetype = qe (quantum espresso bands.pwo)
               eigenval (VASP EIGENVAL file)
               data (wannier90 band.dat, wantools bulkek.dat, QE bands.dat.gnu)
    vectors = np.array([[a1,a2,a3],...,[c1,c2,c3]])
              Real space lattice vectors in order to convert VASP K points (in crystal coord) to cartesian coord
    """
    filetype=filetype.lower()

    if filetype=="data" or filetype==None:
        data=np.loadtxt(fname=filename)
        data=data[:,:2]         #select the first two columns to process (for wannier tools)
        rows=1
        position=data[rows,0]
        while position!=0:     #counter will tell me how many points are in the x axis, number of rows
            rows=rows+1
            position=data[rows,0]
        columns=np.int_(2*data.shape[0]/rows)
        data=np.reshape(data,(rows,columns),order='F')
        final_columns=np.int_(columns/2-1)
        data=np.delete(data,np.s_[0:final_columns],1)

    if filetype[:2]=="qe":
        file=open(filename,'r')
        lines=file.readlines()
        for i,line in enumerate(lines):
            #Grep number of bands
            if re.search('number of Kohn-Sham',line):
                num_bands=int(line.split('=')[1])
            if re.search('number of k points',line):
                num_points=int(line.split()[4])
            if re.search('End of band structure calculation',line):
                results_line=i+1
                break
        data=np.zeros([num_points,num_bands+1])
        data_lines=lines[results_line:]
        dist=0
        i=-1
        j=1
        coord0=np.zeros(3)
        for line in data_lines:
            if re.search('Writing output',line):    #Reading is completed
                break
            elif re.search('bands \(ev\)',line):    #New k_point
                if '-' in line:
                    l=__insert_space_before_minus(line)
                    l=l.split()
                else:
                    l=line.split()
                point1=np.array(l[2:5]).astype(np.float_)
                coord1=point1              # Already in reciprocal cartesian coord (not like VASP)
                delta=np.linalg.norm(coord1-coord0)
                if delta>=0.25:                                  #fast fix for broken paths
                    delta=0
                dist=dist+delta
                coord0=coord1
                i=i+1
                j=1
                data[i,0]=dist
            else:                           #Load energies
                l=line.split()
                energies=np.array(l).astype(np.float_)
                data[i,j:j+len(energies)]=energies
                j=j+len(energies)
       
    elif filetype=="eigenval":
        file=open(filename,'r')
        lines=file.readlines()
        num_points=int(lines[5].split()[1])
        num_bands=int(lines[5].split()[2])
        data_lines=lines[7:]

        data=np.zeros([num_points,num_bands+1])
        if vectors.all()!=None:                      #If there is no cell in the input
            vectors = vectors/np.linalg.norm(vectors[0])
            K_vec=ut.K_basis(vectors)
        else:
            K_vec=np.identity(3)
            print('CAUTIION: There was no cell introduced, threfore distances are wrong')
        dist=0
        i=0
        if K_vec[0].size == 2:
            coord0=np.zeros(2)
            for num in range(0,len(data_lines),num_bands+2):    #load the x position
                line=data_lines[num]
                line=line.split()
                point1=np.array(line).astype(np.float_)[0:2]
                coord1=ut.cryst2cartesian(point1,K_vec)
                delta=np.linalg.norm(coord1-coord0)
                if delta>=0.25:                                  #fast fix for broken paths
                    delta=0
                dist=dist+delta
                data[i,0]=dist
                coord0=coord1
                i=i+1
        else:
            coord0  =np.zeros(3)
            for num in range(0,len(data_lines),num_bands+2):    #load the x position
                line=data_lines[num]
                line=line.split()
                point1=np.array(line).astype(np.float_)[0:3]
                coord1=ut.cryst2cartesian(point1,K_vec)
                delta=np.linalg.norm(coord1-coord0)
                if delta>=0.25:                                  #fast fix for broken paths
                    delta=0
                dist=dist+delta
                data[i,0]=dist
                coord0=coord1
                i=i+1
        for band in range(1,num_bands+1):                             #load the bands
            i=0
            for num in range(band,len(data_lines),num_bands+2):
                line=data_lines[num]
                line=line.split()
                data[i,band]=line[1]
                i=i+1
        data[:,0]=data[:,0]-data[0,0]
    return data

def __plot_electrons(file,filetype=None,vectors=np.array(None),ticks=np.array(None),fermi=None,color=None,style=None,marker=None,legend=None,num_elec=None,save_raw_data=None,ax=None):
    """Print the bands given by __process_electron_bands 
    BUT DOES NOT SHOW THE OUTPUT (not plt.show())
    file = Path to the file
    filetype = qe (quantum espresso bands.pwo)
               eigenval (VASP EIGENVAL file)
               data (wannier90 band.dat, wantools bulkek.dat, QE bands.dat.gnu)
    vectors = np.array([[a1,a2,a3],...,[c1,c2,c3]])
              Real space lattice vectors in order to convert VASP K points (in crystal coord) to cartesian coord
    ticks = np.array([[tick1x,tick1y,tick1z],...,[ticknx,tickny,ticknz]])
    fermi = Fermi energy in order to shift the band structure and put it at zero
    color = string with the color for the bands or 'VC' and num_electrons to use blue/red for valence/conduction
    num_elec= Number of electrons
    style = string with the linestyle (solid, dashed, dotted)
    save_raw_data = 'File to save the plotable data'
    ax = ax in which to plot
    """
    data=__process_electron_bands(file,filetype,vectors)
    if save_raw_data != None:
        np.savetxt(save_raw_data,data)

    if fermi!=None:                       #Fermi energy
        data[:,1:]=data[:,1:]-fermi

    if vectors.any()!=None and ticks.any()!=None:    #ticks and labels
        ticks=__ticks_generator(vectors,ticks)
        x_lim=ticks[ticks.shape[0]-1]                #resize x_data for wannier and other codes output
        data[:,0]=data[:,0]*(x_lim/data[:,0].max())

    if color=='VC':
        ax.plot(data[:,0],data[:,1],linestyle=style,marker=marker,linewidth=0.7,color='tab:blue',label=legend)
        ax.plot(data[:,0],data[:,2:num_elec+1],linestyle=style,marker=marker,linewidth=0.7,color='tab:blue')
        ax.plot(data[:,0],data[:,num_elec+1:],linestyle=style,marker=marker,linewidth=0.7,color='tab:red')
    else:
        ax.plot(data[:,0],data[:,1],linestyle=style,marker=marker,linewidth=0.7,color=color,label=legend)
        ax.plot(data[:,0],data[:,2:],linestyle=style,marker=marker,linewidth=0.7,color=color)


    delta_y=data[:,1:].max()-data[:,1:].min()
    #returns max and min values for y and x of the data sample (usefull for plotting)
    #x_min,x_max,y_min,y_max
    #plt.show()
    return [data[:,0].min(),data[:,0].max(),data[:,1:].min(),data[:,1:].max()]

def bands(file,KPATH=None,aux_file=None,title=None,vectors=np.array(None),ticks=np.array(None),labels=None,
               fermi=None,window=None,num_elec=None,color=None,filetype=None,figsize=None,save_as=None,save_raw_data=None,axis=None):
    """Plots the:
        bands.pwo file of a band calculation in Quantum Espresso
        EIGENVALUES file of a VASP calculation
        bands.dat.gnu file of bands postprocessing (Quantum Espresso)
        band.dat file in Wannier90
        bulkek.dat in Wanniertools

    Minimal plots can be done with just:
        file = Path to the file with bandstructure
        filetype = (It should be detected automatically)
                   qe (quantum espresso bands.pwo)
                   EIGENVAL (VASP EIGENVAL file)
                   data (wannier90 band.dat, wantools bulkek.dat, QE bands.dat.gnu)

    Two aditional files can be provide to autocomplete almost everything:
        KPATH = File with PATH and legends for the HSP in the VASP format as provided by topologicalquantumchemistry.fr
        aux_file = A file from which read the Fermi level, number of electrons and structure.
                   In the case of QE this would be an scf.pwo of nscf.pwo
                   In the case of VASP this is the OUTCAR

    However everything may be introduced manually:

    vectors = np.array([[a1,a2,a3],...,[c1,c2,c3]])
              Real space lattice vectors in order to convert VASP K points (in crystal coord) to cartesian coord
    ticks = np.array([[tick1x,tick1y,tick1z],...,[ticknx,tickny,ticknz]])
            Ticks in your bandstructure (the usual HSP)
    labels = ["$\Gamma$","$X$","$M$","$\Gamma$"]
    title = 'Your nice and original title for the plot'
    fermi = Fermi energy in order to shift the band structure accordingly
    window = Window of energies to plot around Fermi, it can be a single number or 2
            either window=0.5 or window=[-0.5,0.5] => Same result
    color = Either a color or "VC" to use Valence and Conduction bands with different color
    figsize = (int,int) => Size and shape of the figure
    save_as = 'wathever.format'
    save_raw_data = 'file.dat' The processed data ready to plot
    axis = ax in which to plot, if no axis is present new figure is created
    """
    #READ input
    if filetype == None:
        file = ut.file(file)
    else:
        file = ut.file(file,filetype)
    if KPATH != None:
        KPATH=ut.file(KPATH)
    if aux_file != None:
        aux_file=ut.file(aux_file)

    #Select parameters
    if KPATH!=None:
        ticks,labels=KPATH.path,KPATH.labels
    if file.filetype[:2]=='qe' and aux_file==None:
        vectors=file.lattice

    if aux_file!=None:
        v=aux_file.lattice
        f=aux_file.fermi
        n=aux_file.electrons
        if fermi==None:
            fermi=f
        if num_elec==None:
            num_elec=n
        if vectors.any()==None:
            vectors=v

    if fermi!=None:
        if window==None:
            window=1
        if num_elec!=None and color==None:
            color='VC'

    if axis == None:
        fig=plt.figure(figsize=figsize)
        ax = fig.add_subplot(111)
    else:
        ax=axis
    limits=__plot_electrons(file.file,file.filetype,vectors,ticks,fermi,color=color,num_elec=num_elec,save_raw_data=save_raw_data,ax=ax)

    ax.set_ylabel('energy (eV)',labelpad=-1)

    if fermi!=None:                       #Fermi energy
        ax.axhline(y=0,color='black',linewidth=0.4)
        if window!=None:                   #Limits y axis
            if type(window) is int or type(window) is float:
                ax.set_ylim(-window,window)
            elif type(window) is list:
                ax.set_ylim(window[0],window[1])
        else:
            delta_y=limits[3]-limits[2]
            ax.set_ylim(limits[2]-delta_y*0.05,limits[3]+delta_y*0.1)

    if vectors.any()!=None and ticks.any()!=None:    #ticks and labels
        ticks=__ticks_generator(vectors,ticks)
        if labels != None :
            ax.set_xticks(ticks,labels)
        else:
            ax.set_xticks(ticks)
        for i in range(1,ticks.shape[0]-1):
            ax.axvline(ticks[i],color='gray',linestyle='--',linewidth=0.4)
    else:
        ax.set_xticks([])

    if title!=None:                             #Title option
        ax.set_title(title)

    ax.set_xlim(limits[0],limits[1])   #Limits in the x axis
    plt.tight_layout()

    if save_as!=None:                             #Saving option
        plt.savefig(save_as, dpi=500)
    if axis == None:
        plt.show()

def bands_compare(files,KPATH=None,fermi=None,legends=None,title=None,aux_file=None,vectors=np.array(None),
                  ticks=np.array(None),labels=None,window=1,figsize=None,save_as=None,filetypes=None,
                  styles=['-','--','-.',':'],
                  markers=['','','',''],
                  colors=['tab:blue','tab:red','tab:green','tab:orange'],
                  axis=None):
    """Plots the:
        bands.pwo file of a band calculation in Quantum Espresso
        EIGENVALUES file of a VASP calculation
        bands.dat.gnu file of bands postprocessing (Quantum Espresso)
        band.dat file in Wannier90
        bulkek.dat in Wanniertools

    Minimal plots can be done with just:
        files = list of files with bandstructures 
        filetypes = list with the filetype for each file (It should be detected automatically)
                   qe (quantum espresso bands.pwo)
                   EIGENVAL (VASP EIGENVAL file)
                   data (wannier90 band.dat, wantools bulkek.dat, QE bands.dat.gnu)

    Four aditional parameters can be provide to autocomplete almost everything:
        KPATH = File with PATH and legends for the HSP in the VASP format as provided by topologicalquantumchemistry.fr
        fermi = list with the different Fermi energies
        legends = List with the legend for each dataset
        aux_file = File from which to read the structure to correctly represent distances in reciprocal space

    However everything may be introduced manually:

    vectors = np.array([[a1,a2,a3],...,[c1,c2,c3]])
              Real space lattice vectors in order to convert VASP K points (in crystal coord) to cartesian coord
    ticks = np.array([[tick1x,tick1y,tick1z],...,[ticknx,tickny,ticknz]])
            Ticks in your bandstructure (the usual HSP)
    labels = ["$\Gamma$","$X$","$M$","$\Gamma$"]
    title = 'Your nice and original title for the plot'
    window = Window of energies to plot around Fermi, it can be a single number or 2
            either window=0.5 or window=[-0.5,0.5] => Same result
    colors = list with colors
    styles = list with style lines (solid, dashed, dotted...)
    figsize = (int,int) => Size and shape of the figure
    save_as = 'wathever.format'
    axis = ax in which to plot, if no axis is present new figure is created
    """
    styles=styles*4
    colors=colors*4
    markers=markers*4

    #READ input
    if filetypes == None:
        filetypes = [None] * len(files)
        for i in range(len(filetypes)):
            filetypes[i]=ut.grep_filetype(files[i])
    else:
        for i in range(len(filetypes)):
            filetypes[i]=filetypes[i].lower()
    if KPATH != None:
        KPATH=ut.file(KPATH)
    if aux_file != None:
        aux_file=ut.file(aux_file)

    #Select parameters
    if fermi==None:
        fermi=np.zeros(len(files))
    if legends==None:
        legends=['Data ' + str(n+1) for n in range(len(files))]
    if KPATH!=None:
        ticks,labels=KPATH.path,KPATH.labels
    for i,kind in enumerate(filetypes):            #Automatically read vectors if possible
        if kind[:2]=='qe' and aux_file==None and vectors.any()==None:
            vectors=ut.grep_lattice(files[i])
            break
    if aux_file!=None and vectors.any()==None:
        vectors=aux_file.lattice
    
    if axis == None:
        fig = plt.figure(figsize=figsize)
        ax = fig.add_subplot(111)
    else:
        ax = axis
    for num in range(len(files)):
        data_limits=__plot_electrons(files[num],filetypes[num],vectors,ticks,fermi=fermi[num],color=colors[num]
                                       ,style=styles[num],marker=markers[num],legend=legends[num],ax=ax)
        if num==0:
            limits=data_limits
        else:
            limits[0]=min(data_limits[0],limits[0])
            limits[1]=max(data_limits[1],limits[1])
            limits[2]=min(data_limits[2],limits[2])
            limits[3]=max(data_limits[3],limits[3])
        
    ax.set_ylabel('energy (eV)',labelpad=-1)
    ax.axhline(y=0,color='black',linewidth=0.4) #Fermi energy (mandatory, if not comparison has no sense)
    
    if type(window) is int or type(window) is float: #window of energyes to plot
        ax.set_ylim(-window,window)
    elif type(window) is list:
        ax.set_ylim(window[0],window[1])
    else:
        delta_y=limits[3]-limits[2]
        ax.set_ylim(limits[2]-delta_y*0.05,limits[3]+delta_y*0.1)
    
    if vectors.any()!=None and ticks.any()!=None:    #ticks and labels
        ticks=__ticks_generator(vectors,ticks)
        if labels != None :
            ax.set_xticks(ticks,labels)
        else:
            ax.set_xticks(ticks)
        for i in range(1,ticks.shape[0]-1):
            ax.axvline(ticks[i],color='gray',linestyle='--',linewidth=0.4)
    else:
        ax.set_xticks([])
    
    if title!=None:                             #Title option
        ax.set_title(title)
    
    ax.set_xlim(limits[0],limits[1])   #Limits in the x axis
    
    ax.legend()
    if save_as!=None:                             #Saving option
        plt.savefig(save_as, dpi=500)
    if axis == None:
        plt.show()


# PLOTTING PHONONS----------------------------------------------------------------

def __lineseg_dist(p, a, b):
    """Function lineseg_dist returns the distance the distance from point p to line segment [a,b]. p, a and b are np.arrays."""
    # normalized tangent vector
    d = np.divide(b - a, np.linalg.norm(b - a))
    # signed parallel distance components
    s = np.dot(a - p, d)
    t = np.dot(p - b, d)
    # clamped parallel distance
    h = np.maximum.reduce([s, t, 0])
    # perpendicular distance component
    c = np.cross(p - a, d)
    return np.hypot(h, np.linalg.norm(c))

def __process_phonon_bands(gnu_file,QE_path):
    if QE_path.any()==None:
        data=np.loadtxt(fname=gnu_file)
    else:
        #SELECT THE LINES WHERE THE PATH IS SPLITED
        lines=[]
        line=0
        for i in range(len(QE_path)):
            if QE_path[i,3]==1:
                lines=lines+[line]
            line=line+int(QE_path[i,3])

        #LOAD THE DATA AND CORRECT THE SPLITS TO MAKE IT "CONTINOUS"
        data=np.loadtxt(gnu_file)
        for line in lines:
            if line < data.shape[0]-1:
                data[line+1:,0]=data[line+1:,0]-(data[line+1,0]-data[line,0])
    return data

def __plot_phonons(file,linewidth,vectors=np.array(None),ticks=np.array(None),
                        color=None,style=None,legend=None,QE_path=None,ax=None):
    """Print the phonons.freq.gp file of matdyn output (Quantum Espresso)
    BUT DOES NOT SHOW THE OUTPUT (not plt.show())
    plot_phonons(file,real_vecs,ticks)
    file=Path to the file
    vectors=np.array([[a1,a2,a3],...,[c1,c2,c3]])
    ticks=np.array([[tick1x,tick1y,tick1z],...,[ticknx,tickny,ticknz]])
    color = string with the color for the bands
    style = string with the linestyle (solid, dashed, dotted)
    legend = legend to add for the data set
    QE_path = matdyn file for splitted paths, it will correct the path to make it "continous"
    QE_path = reciprocal space path as given by the grep_ticks_QE or grep_ticks_labels_KPATH. It will fix splitted paths and correct the path to make it "continous"
    ax = ax in which to plot
    """
    data=__process_phonon_bands(file,QE_path)

    if vectors.any()!=None and ticks.any()!=None:    #ticks and labels
        ticks=__ticks_generator(vectors,ticks)
        x_lim=ticks[ticks.shape[0]-1]                #resize x_data for wannier and other codes output
        data[:,0]=data[:,0]*(x_lim/data[:,0].max())

    ax.plot(data[:,0],data[:,1],linestyle=style,linewidth=linewidth,color=color,label=legend)
    ax.plot(data[:,0],data[:,2:],linestyle=style,linewidth=linewidth,color=color)

    return [data[:,0].min(),data[:,0].max(),data[:,1:].min(),data[:,1:].max()]

def phonons(file,KPATH=None,ph_out=None,title=None,matdyn_in=None,grid=True,vectors=np.array(None),
                ticks=np.array(None),labels=None,save_as=None,figsize=None,color=None,linewidth=0.7,axis=None):
    """Plots phonon spectra provided by Quantum Espresso output 
    (it supports discontinous paths and highlights the computed points)
    Minimal plots can be done with just:
        file = Path to the freq.gp file provided by matdyn

    Three aditional files can be provide to autocomplete almost everything:
        KPATH = File with PATH and legends for the HSP in the VASP format as provided by topologicalquantumchemistry.fr
        ph_out = ph.x output (Needed to read lattice parameters and grid where the phonons are computed)
        matdyn_in = matdyn subroutine input (in order to read the KPATH and correct for discontinous paths)

    However everything may be introduced manually:
    title = 'Your nice and original title for the plot'
    grid = Bolean (Whether you want the grid to be highlighted in pink)
    vectors = np.array([[a1,a2,a3],...,[c1,c2,c3]])
              Real space lattice vectors in order to convert VASP K points (in crystal coord) to cartesian coord
    ticks=np.array([[tick1x,tick1y,tick1z],...,[ticknx,tickny,ticknz]])
    labels=["$\Gamma$","$X$","$M$","$\Gamma$"]
    save_as='wathever.format'
    figsize = (int,int) => Size and shape of the figure
    color = color for your plot
    linewidth = linewidth of your plot
    axis = ax in which to plot, if no axis is present new figure is created
    """

    if KPATH!=None:
        ticks,labels=ut.grep_ticks_labels_KPATH(KPATH)
    if matdyn_in!=None:
        ticks=ut.grep_ticks_QE(matdyn_in)
    if ph_out!=None and vectors.any()==None:
        vectors=ut.grep_lattice(ph_out)
    if axis == None:
        fig=plt.figure(figsize=figsize)
        ax = fig.add_subplot(111)
    else:
        ax=axis

    limits=__plot_phonons(file,linewidth,vectors,ticks,color=color,QE_path=ticks,ax=ax)

    ax.set_ylabel('frequency $(cm^{-1})$')
    ax.axhline(y=0,color='gray',linestyle='--',linewidth=0.4)

    ax.set_xlim(limits[0],limits[1])   #Limits in the x axis
    delta_y=limits[3]-limits[2]
    ax.set_ylim(limits[2]-delta_y*0.05,limits[3]+delta_y*0.1) #Limits in the y axis

    if vectors.any()!=None and ticks.any()!=None:    #ticks and labels
        if ph_out!=None and grid==True:
            grid_points=ut.grep_ph_grid_points(ph_out,expanded=True,decimals=15) #to correctly find distances
            ticks,grid=__ticks_generator(vectors,ticks,grid_points)
            ax.set_xticks(ticks,labels)
            for point in grid:
                for i in range(len(ticks)):
                    if point==ticks[i]:
                        ticks=np.delete(ticks,i)
                        break
            for i in range(len(grid)):
                ax.axvline(grid[i],color='deeppink',linestyle='-.',linewidth=0.5)
        else:
            ticks=__ticks_generator(vectors,ticks)
            if labels != None:
                ax.set_xticks(ticks,labels)
            else:
                ax.set_xticks(ticks)
        for i in range(len(ticks)):
            ax.axvline(ticks[i],color='gray',linestyle='--',linewidth=0.4)
    else:
        ax.set_xticks([])

    if title!=None:                             #Title option
        ax.set_title(title)

    plt.tight_layout()
    if save_as!=None:                             #Saving option
        plt.savefig(save_as, dpi=500)
    if axis == None:
        plt.show()

def phonons_compare(files,KPATH=None,ph_outs=None,legends=None,title=None,matdyn_in=None,grid=True,
                         vectors=np.array(None),ticks=np.array(None),labels=None,save_as=None,
                         colors=['tab:blue','tab:red','tab:green','tab:orange'],
                         styles=['solid','dashed','dashdot','dotted'],linewidth=0.7,figsize=None,axis=None):
    """Plots phonon spectra provided by Quantum Espresso output 
    (it supports discontinous paths and highlights the computed points)
    Minimal plots can be done with just:
        files = List of paths to the freq.gp file provided by matdyn

    Four aditional parameters can be provide to autocomplete almost everything:
        KPATH = File with PATH and legends for the HSP in the VASP format as provided by topologicalquantumchemistry.fr
        ph_out = List of ph.x output files (Needed to read lattice parameters and grid where the phonons are computed)
        matdyn_in = matdyn subroutine input (in order to read the KPATH and correct for discontinous paths)
        legends = List with the legend for each dataset

    However everything may be introduced manually:
    title = 'Your nice and original title for the plot'
    grid = Bolean (Whether you want the grid to be highlighted in pink)
    vectors = np.array([[a1,a2,a3],...,[c1,c2,c3]])
              Real space lattice vectors in order to convert VASP K points (in crystal coord) to cartesian coord
    ticks=np.array([[tick1x,tick1y,tick1z],...,[ticknx,tickny,ticknz]])
    labels=["$\Gamma$","$X$","$M$","$\Gamma$"]
    save_as='wathever.format'
    figsize = (int,int) => Size and shape of the figure
    linewidth = linewidth of your plot
    colors = list with colors
    styles = list with style lines (solid, dashed, dotted...)
    axis = ax in which to plot, if no axis is present new figure is created
    """
    styles=styles*4
    colors=colors*4

    if legends==None:
        legends=['Data ' + str(n+1) for n in range(len(files))]
    if KPATH!=None:
        ticks,labels=ut.grep_ticks_labels_KPATH(KPATH)
    if matdyn_in!=None:
        ticks=ut.grep_ticks_QE(matdyn_in)
    if ph_outs!=None and vectors.any()==None:
        vectors=ut.grep_lattice(ph_outs[0])

    if axis == None:
        fig=plt.figure(figsize=figsize)
        ax = fig.add_subplot(111)
    else:
        ax=axis

    for num in range(len(files)):
        data_limits=__plot_phonons(files[num],linewidth,vectors,ticks,color=colors[num]
                                       ,style=styles[num],legend=legends[num],QE_path=ticks,ax=ax)
        if num==0:
            limits=data_limits
        else:
            limits[0]=min(data_limits[0],limits[0])
            limits[1]=max(data_limits[1],limits[1])
            limits[2]=min(data_limits[2],limits[2])
            limits[3]=max(data_limits[3],limits[3])

    ax.set_ylabel('frequency $(cm^{-1})$')
    ax.axhline(y=0,color='gray',linestyle='--',linewidth=0.4)

    ax.set_xlim(limits[0],limits[1])   #Limits in the x axis
    delta_y=limits[3]-limits[2]
    ax.set_ylim(limits[2]-delta_y*0.05,limits[3]+delta_y*0.1) #Limits in the y axis

    if vectors.any()!=None and ticks.any()!=None:    #ticks and labels
        if ph_outs!=None and grid==True:
            grid_points=ut.grep_ph_grid_points(ph_outs[0],expanded=True,decimals=15)
            path=ticks
            ticks,grid=__ticks_generator(vectors,ticks,grid_points)
            ax.set_xticks(ticks,labels)
            for num in range(len(ph_outs)):
                grid_points=ut.grep_ph_grid_points(ph_outs[num],expanded=True,decimals=15)
                grid=__ticks_generator(vectors,path,grid_points)[1]
                for point in grid:
                    for i in range(len(ticks)):
                        if point==ticks[i]:
                            ticks=np.delete(ticks,i)
                            break
                for i in range(len(grid)):
                    if len(ph_outs)==1:
                        ax.axvline(grid[i],color='deeppink',linestyle='-.',linewidth=0.5)
                    else:
                        ax.axvline(grid[i],color=colors[num],linestyle=styles[num],linewidth=0.5)
        else:
            ticks=__ticks_generator(vectors,ticks)
            if labels != None:
                ax.set_xticks(ticks,labels)
            else:
                ax.set_xticks(ticks)
        for tick in ticks:
            ax.axvline(tick,color='gray',linestyle='--',linewidth=0.4)
    else:
        ax.set_xticks([])

    if title!=None:                             #Title option
        ax.set_title(title)
    ax.legend()
    plt.tight_layout()
    if save_as!=None:                             #Saving option
        plt.savefig(save_as, dpi=500)
    if axis == None:
        plt.show()



#PYTHON module with nice tu have utilitary tools, mostly focussed on text scraping

import numpy as np
import re
from ase import io

import yaiv.constants as const

# GREPPING utilities----------------------------------------------------------------

class file:
    """A class for file scraping, depending on the filetype a different set of attributes will initialize.
    The filetype should be automatically detected, but can be manually introduced:
    QuantumEspresso: qe_scf_in, qe_scf_out, qe_bands_in, qe_ph_out, matdyn_in
    VASP: POSCAR, OUTCAR, KPATH (KPOINTS in line mode), EIGENVAL
    """
    def __init__(self,file,filetype=None):
        self.file = file
        #Define file type
        if filetype == None:
            self.filetype = grep_filetype(file)
        else:
            self.filetype = filetype.lower()
        #Read attributes:
        if self.filetype in ['qe_scf_out','qe_scf_in','qe_bands_in','qe_ph_out','outcar','poscar']:
            self.lattice = grep_lattice(self.file,filetype=self.filetype)
        if self.filetype in ['qe_scf_out','outcar']:
            self.electrons = grep_electrons(file,filetype=self.filetype)
        if self.filetype in ['qe_scf_out','outcar']:
            self.fermi = grep_fermi(file,filetype=self.filetype)
        if self.filetype == 'kpath':
            self.path,self.labels = grep_ticks_labels_KPATH(file)
        if self.filetype in ['qe_bands_in','matdyn_in']:
            self.path = grep_ticks_QE(self.file,self.filetype)
    def __str__(self):
        return str(self.filetype) + ':\n' + self.file
    def grep_lattice(self,alat=False):
        """Check grep_lattice function"""
        self.lattice = grep_lattice(self.file,filetype=self.filetype,alat=alat)
        return self.lattice
    def reciprocal_lattice(self,alat=False):
        """Check K_basis function"""
        if hasattr(self, 'lattice'):
            return K_basis(self.lattice,alat=alat)
        else:
            print('No lattice data in order to compute reciprocal lattice')
    def grep_ph_grid_points(self,expanded=False,decimals=3):
        """Check grep_ph_grid_points function"""
        if self.filetype != 'qe_ph_out':
            print('This method if for ph.x outputs, which this is not...')
            print('Check the documentation for grep_ph_grid_points function')
        else:
            grid = grep_ph_grid_points(self.file,expanded=expanded,decimals=decimals)
            self.ph_grid_points = grid
            return grid
    def grep_total_energy(self,meV=False):
        """Returns the total energy in (Ry). Check grep_total_energy"""
        out= grep_total_energy(self.file,meV=meV,filetype=self.filetype)
        self.total_energy = out
        return out

def grep_filetype(file):
    """Returns the filetype, currently it supports:
    QuantumEspresso: qe_scf_in, qe_scf_out, qe_bands_in, qe_ph_out, matdyn_in
    VASP: POSCAR, OUTCAR, KPATH (KPOINTS in line mode), EIGENVAL
    Anything else is considered a general 'data' type
    """
    lines = open(file)
    counter=0
    for line in lines:
        if re.search('calculation.*scf.*',line,re.IGNORECASE) or re.search('calculation.*nscf.*',line,re.IGNORECASE):
            filetype='qe_scf_in'
            break
        elif re.search('Program PWSCF',line,re.IGNORECASE):
            filetype='qe_scf_out'
            break
        elif re.search('Program PHONON',line,re.IGNORECASE):
            filetype='qe_ph_out'
            break
        elif re.search('calculation.*bands.*',line,re.IGNORECASE):
            filetype='qe_bands_in'
            break
        elif re.search('flfrc',line,re.IGNORECASE):
            filetype='matdyn_in'
            break
        elif re.search('direct',line,re.IGNORECASE) or re.search('cartesian',line,re.IGNORECASE):
            filetype='poscar'
            break
        elif re.search('vasp',line,re.IGNORECASE):
            filetype='outcar'
            break
        elif len(line.split()) == 4 and all([x.isdigit() for x in line.split()]):
            filetype='eigenval' 
            break
        elif re.search('line.mode',line,re.IGNORECASE):
            filetype='kpath' 
            break
        else:
            filetype='data'
    return filetype

def grep_lattice(file,alat=False,filetype=None):
    """Greps the lattice vectors from a variety of outputs (it uses ase)

    alat = Bolean controling if you want your lattice normalized (mod(a0) = 1, alat units)
   
    The filetype should be given by the function grep_filetype(file)

    OUTPUT= np.array([vec1,vec2,vec3])
    """
    if filetype == None:
        filetype = grep_filetype(file)
    else:
        filetype = filetype.lower()
    if filetype=='qe_ph_out':
        lattice_lines=False
        lines=open(file,'r')
        for line in lines:
            if re.search('lattice parameter',line):
                line=line.split()
                alat_au=float(line[4])
            elif lattice_lines==True:
                X=float(line.split()[3])
                Y=float(line.split()[4])
                Z=float(line.split()[5])
                vec=np.array([X,Y,Z])
                try:
                    lattice=np.vstack((lattice,vec))            
                except NameError:
                    lattice=vec
                if lattice.shape == (3,3):
                    break
            elif re.search('crystal axes',line,flags=re.IGNORECASE):
                lattice_lines=True
        if alat == True:
            return lattice
        else:
            return lattice*alat_au*const.au2ang
    else:
        import warnings
        warnings.filterwarnings("ignore", message="Non-collinear spin is not yet implemented. Setting magmom to x value.")
        try:
            data=io.read(file)
            lattice=np.array(data.cell)
            if alat == True:
                lattice = lattice/np.linalg.norm(lattice[0])
        except:
            lattice=None
            print('No lattice data found')
    return lattice

def grep_fermi(file,filetype=None):
    """Greps the Fermi level from a variety of filetypes and returns it in eV
    The filetype should be detected automatically, but it supports:
    qe_scf_out (Quantum Espresso), OUTCAR (VASP)
    """
    E_f=None
    if filetype == None:
        filetype = grep_filetype(file)
    else:
        filetype=filetype.lower()
    if filetype[:2]=='qe':
        scf_out=open(file,'r')
        for line in scf_out:
            if re.search('Fermi energy is',line):
                E_f=float(line.split()[4])
            if re.search('highest occupied',line):
                if re.search('unoccupied',line):
                    E1=float(line.split()[6])
                    E2=float(line.split()[7])
                    print('The gap is',(E2-E1)*1000,'meV')
                    E_f=E1+(E2-E1)/2
                else:
                    E_f=float(line.split()[4])
    elif filetype=='outcar':
        OUTCAR=open(file,'r')
        for line in OUTCAR:
            if re.search('E-fermi',line):
                E_f=float(line.split()[2])
    return E_f

def grep_electrons(file,filetype=None):
    """Greps the number of electrons from a scf.pwo or OUTCAR file.
    The filetype should be detected automatically, but it supports:
    qe_scf_out (Quantum Espresso), OUTCAR (VASP)
    """
    num_elec=None
    if filetype == None:
        filetype = grep_filetype(file)
    else:
        filetype=filetype.lower()
    if filetype[:2]=='qe':
        scf_out=open(file,'r')
        for line in scf_out:
            if re.search('number of electrons',line):
                num_elec=int(float(line.split()[4]))
    elif filetype=='outcar':
        OUTCAR=open(file,'r')
        for line in OUTCAR:
            if re.search('NELECT',line):
                num_elec=int(float(line.split()[2]))
    return num_elec

def grep_ticks_labels_KPATH(file):
    """Greps ticks and labels of the ticks from a KPATH file of VASP.
    It expects the file to have the structure:
    0 0 0 !GM
    0 0.5 0 !X
    0 0.5 0 !X
    0.5 0.5 0 !

    It outputs two variables, the PATH:
    np.array([K-point1, # of points to next],
             [K-point2, # of points to next],
              ...]


    and a list of LABELS:
    [label1, label2, label3 ...]
    """
    KPATH=open(file,'r')
    ticks=np.zeros(0)
    q0=np.zeros(0)
    repeat=False
    labels=[]
    path_section=False
    for line in KPATH:
        if path_section==True:
            if len(line.split())!=0:
                X=float(line.split()[0])
                Y=float(line.split()[1])
                Z=float(line.split()[2])
                q1=np.array([X,Y,Z,100])
                label=line.split('!')[1].split()[0]
                if not np.array_equal(q0,q1):
                    labels=labels+[label]
                    if len(ticks)==0:
                        ticks=q1
                    else:
                        if repeat==False:
                            if len(ticks.shape)>1:
                                if ticks[len(ticks)-2,3]!=1:
                                    ticks[len(ticks)-1,3]=1
                            ticks=np.vstack((ticks,q1))
                            q0=q1
                        else:
                            ticks=np.vstack((ticks,q1))
                            q0=q1
                            repeat=False
                else:
                    repeat=True
        if re.search('Reciprocal',line,flags=re.IGNORECASE):
            path_section=True
    ticks[len(ticks)-1,3]=1

    num_q=len(ticks[:,0])

    num_labels=num_q
    path=True
    for i in range(num_q):
        if path==False:
            diff=ticks[i,:3]-ticks[i-1,:3]
            labels[i-1]=labels[i-1]+'|'+labels[i]
            labels[i]='000'
        if ticks[i,3]==1:
            path=False
            num_labels=num_labels-1
        else:
            path=True
    while labels.count('000')>0:
        labels.remove('000')
    for i in range(len(labels)):
        if 'Gamma' in labels[i]:
            labels[i]=labels[i].split('Gamma')[0]+'\Gamma'+labels[i].split('Gamma')[1]
        labels[i]='$'+labels[i]+'$'
    return ticks, labels

def grep_ticks_QE(file,filetype=None,silent=True):
    """Greps the K-path from a qe_bands_in or matdyn.in Quantum Espresso files.
    OUTPUT= np.array([K-point1, # of points to next],
                     [K-point2, # of points to next],
                     ...]
    """
    if filetype == None:
        filetype = grep_filetype(file)
    else:
        filetype=filetype.lower()
    KPATH=open(file)
    ticks=np.zeros(0)
    num_q=0
    path_section=False
    for line in KPATH:
        if path_section==True:
            if num_q==0:
                num_q=int(line)
                i=0
            elif i<num_q:
                if len(line.split())!=0:
                    X=float(line.split()[0])
                    Y=float(line.split()[1])
                    Z=float(line.split()[2])
                    points=int(line.split()[3])
                    q1=np.array([X,Y,Z,points])
                    if len(ticks)==0:
                        ticks=q1
                    else:
                        ticks=np.vstack((ticks,q1))
                i=i+1
            else:
                break
        elif re.search('K_POINTS.*crystal_b',line,flags=re.IGNORECASE):
            path_section=True
        elif re.search('/',line) and filetype=='matdyn_in':
            path_section=True
    num_labels=num_q
    path=True
    for i in range(num_q):
        if ticks[i,3]==1:
            path=False
            num_labels=num_labels-1
        else:
            path=True
    if silent == False:
        print("you need to introduce",num_labels+1,"labels")
    return ticks

def grep_ph_grid_points(file,expanded=False,decimals=3):
    """Greps the grid points from a ph.pwo file, it reads the points and the star of those
    points given in the QE ouput and expresses them in reciprocal space lattice vectors. This grid
    can be further expanded to equivalent points line (0,0,0.5) and (0,0,-0.5).

    file='material.ph.pwo'
    """
    vectors=grep_lattice(file,alat=True,filetype='qe_ph_out')
    text=open(file)
    read_text=False
    for line in text:
        if read_text==True and line.split()[0]==str(i):
            split=(line.split())
            point=np.array([float(split[1]),float(split[2]),float(split[3])])
            point=np.matmul(point,vectors.transpose())
            try:
                grid_points=np.vstack([grid_points,point])
            except NameError:
                grid_points=point
            i=i+1
            if i > num_star:
                read_text=False
        elif re.search('Number of q in the star',line):
            num_star=int(line.split()[7])
            i=1
            read_text=True
        elif re.search('In addition there is the',line):
            i=1
            read_text=True
    grid_points=np.around(grid_points,decimals=decimals) #Fix for detecting the grid in the paths
    
    if expanded==True:
        initial_grid=grid_points
        for point in initial_grid:
            expanded_star=__expand_star(point)
            try:
                grid=np.vstack([grid,expanded_star])
            except NameError:
                grid=expanded_star
    else:
        grid=grid_points
    return grid

def __expand_star(q_point):
    """Expands the "star" of each point to equivalent points inside (111) (related by lattice) in the border of the BZ.
    This is usefull when a High Sym point is the (0.5,0,0), this generates (-0.5,0,0), which is not in the star
    since is it the same point.    
    """
    output=[q_point]
    for i in range(3):
        for point in output:
            related1=np.array(point)
            related2=np.array(point)
            related1[i]=point[i]+1
            related2[i]=point[i]-1
            output=np.vstack([output,related1,related2])
    return output

def grep_total_energy(file,meV=False,filetype=None):
    """Greps the total energy (in Ry or meV) from a Quantum Espresso (.pwo) or VASP (OUTCAR)file.
    returns either the energy or a False boolean if the energy was not found"""
    if filetype == None:
        filetype = grep_filetype(file)
    else:
        filetype=filetype.lower()
    lines=open(file,'r')
    energy=False
    if filetype[:2] == 'qe':
        for line in reversed(list(lines)):
            if re.search('!',line):
                l=line.split()
                energy=float(l[4])
                break
    elif filetype == 'outcar':
        for line in reversed(list(lines)):
            if re.search('sigma->',line):
                l=line.split()
                energy=float(l[-1])
                break
        energy=energy/const.Ry2eV
    if meV==True:
        energy=energy*const.Ry2eV*1000
    return energy

# Transformation tools----------------------------------------------------------------

def K_basis(lattice,alat=False):
    """With basis_vec being in rows, it returns the reciprocal basis vectors in rows
    and units of 2pi"""
    if alat == True:
        lattice=lattice/np.linalg.norm(lattice[0])
    K_vec=np.linalg.inv(lattice).transpose() #reciprocal vectors in rows
    return K_vec

def cartesian2cryst(cartesian,cryst_basis,list_of_vec=False):
    """Goes from cartesian units to cryst units. Either vectors or matrices (also list of vectors)
    cartesian: coordinates or matrix in cartesian units
    cryst_basis: crystaline basis written by rows
    return crystal_coord"""
    if len(np.shape(cartesian))==1 or list_of_vec==True:
        crystal_coord=np.matmul(cartesian,np.linalg.inv(cryst_basis))
    elif len(np.shape(cartesian))==2:
        inv=np.linalg.inv(cryst_basis)
        crystal_coord=np.matmul(np.transpose(inv),cartesian)
        crystal_coord=np.matmul(crystal_coord,np.transpose(cryst_basis))
    return crystal_coord

def cryst2cartesian(cryst,cryst_basis,list_of_vec=False):
    """Goes from crystaling units to cartesian units. Either vectors or matrices (also list of vectors)
    cryst: coordinates or  matrix in cryst units
    cryst_basis: crystaline basis written by rows
    return cartesian_coord"""
    if len(np.shape(cryst))==1 or list_of_vec==True:
        cartesian_coord=np.matmul(cryst,cryst_basis)
    elif len(np.shape(cryst))==2:
        cartesian_coord=np.matmul(np.transpose(cryst_basis),cryst)
        inv=np.linalg.inv(cryst_basis)
        cartesian_coord=np.matmul(cartesian_coord,np.transpose(inv))
    return cartesian_coord

def cartesian2spherical(xyz,degrees=False):
    """From cartesian to spherical coord
    mod, theta(z^x), phi(x^y)"""
    ptsnew = np.zeros(3)
    xy = xyz[0]**2 + xyz[1]**2
    ptsnew[0] = np.sqrt(xy + xyz[2]**2)
    ptsnew[1] = np.arctan2(np.sqrt(xy), xyz[2]) # for elevation angle defined from Z-axis down
    #ptsnew[1] = np.arctan2(xyz[:,2], np.sqrt(xy)) # for elevation angle defined from XY-plane up
    ptsnew[2] = np.arctan2(xyz[1], xyz[0])
    if degrees==True:
        ptsnew[1:]=ptsnew[1:]*180/np.pi
    return ptsnew

def cryst2spherical(cryst,cryst_basis,degrees=False):
    """From crystal coord to spherical (usefull for SKEAF)
    mod, theta(z^x), phi(x^y)"""
    xyz=cryst2cartesian(cryst,cryst_basis)
    spherical=cartesian2spherical(xyz,degrees)
    return spherical



#PYTHON module for convergence analysis

import re
import glob
import numpy as np
import matplotlib.pyplot as plt

import yaiv.constants as const
import yaiv.utils as utils
import yaiv.convergence.cutoff as conv

def read_data(folder):
    """from the folder where data is stored it reads the foo.dyn1 files
    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the (cutoff/smearing) number
    ...
    OUTPUT:
    A list where odd numbers are the Kdrid and even numbers are the corresponding folder info:
    [[  (cutoff/smearing)        freq1  freq1  freq3 ...]
     [   90.        freq1  freq1  freq3 ...]
     [  100.        freq1  freq1  freq3 ...]]

    where first column is the (cutoff/smearing).
    """
    data=[]
    Kgrids=glob.glob(folder+"/*")
    Kgrids=[i.split("/")[-1] for i in Kgrids]
    Kgrids=conv.__sort_Kgrids(Kgrids)
    for i in range(len(Kgrids)):
        grid_data=np.zeros(0)
        Kgrid=Kgrids[i]
        files=glob.glob(folder+"/"+Kgrid+"/*/*dyn1")
        for j in range(len(files)):
            cutoff=files[j].split("/")[-2]
            cutoff_data=np.array([cutoff])
            file = open (files[j],"r")
            for line in file:
                if re.search("freq",line):
                    freq=line.split()[7]
                    cutoff_data=np.append(cutoff_data,freq)
            if grid_data.shape[0]==0:
                grid_data=cutoff_data
            else:
                grid_data=np.vstack((grid_data,cutoff_data))
        data=data+[Kgrid]+[grid_data]
    for i in range(0,len(data),2):
        order=data[i+1][:,0].astype(float).argsort()
        data[i+1]=data[i+1][order]   #sort acording to first column (x axis)(cutoff)
        data[i+1]=data[i+1].astype(float)
    return data

def __get_cmap(n, name='tab10'):
    '''Returns a function that maps each index in 0, 1, ..., n-1 to a distinct
    RGB color; the keyword argument name must be a standard mpl colormap name.'''
    return plt.cm.get_cmap(name, n)

def phonons_vs_cutoff(data,freqs=None,grid=True,save_as=None,axis=None,title=None):
    """It plots the frequencies as a function of cutoff for different k_grids
    
    data = Either the data, or folder where data is stored (it reads the foo.dyn1 files.)
    freqs = Select until which frequency you want to plot (integer).
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.
    title = A title of your liking

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
    else:
        ax=axis

    if freqs==None:
        freqs=len(data[1][0])
    freqs=freqs+1

    cmap=__get_cmap(int(len(data)/2))
    for i in range(0,len(data),2):
        color=cmap(int(i/2))
        plot_data=data[i+1]
        ax.plot(plot_data[:,0],plot_data[:,1],'.-',color=color,label=data[i])
        ax.plot(plot_data[:,0],plot_data[:,2:freqs],'.-',color=color)

    ax.set_ylabel("Freqs (cm-1)")
    ax.set_xlabel("cutoff (Ry)")
    ax.legend(prop={'size': 7})

    if title!=None:
        fig.suptitle(title)
    if grid == True:
        ax.grid()
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()

def phonons_vs_smearing(data,freqs=None,grid=True,temp=False,save_as=None,axis=None,title=None):
    """It plots the frequencies as a function of the smearing for different k_grids
    
    data = Either the data, or folder where data is stored (it reads the foo.dyn1 files.)
    freqs = Select until which frequency you want to plot (integer).
    grid = Bolean that allows for a grid in the plot.
    temp = Bolean if you want a secondary axis with the Temperature equiv of the smearing value (usefull for Fermi-Dirac smearing).
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.
    title = A title of your liking

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the smearing number
    """
    if type(data)==str:
        data=read_data(data)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
    else:
        ax=axis

    if freqs==None:
        freqs=len(data[1][0])
    freqs=freqs+1

    cmap=__get_cmap(int(len(data)/2))
    for i in range(0,len(data),2):
        color=cmap(int(i/2))
        plot_data=data[i+1]
        ax.plot(plot_data[:,0],plot_data[:,1],'.-',color=color,label=data[i])
        ax.plot(plot_data[:,0],plot_data[:,2:freqs],'.-',color=color)

    ax.set_ylabel("Freqs (cm-1)")
    ax.set_xlabel("smearing (Ry)")
    ax.legend(prop={'size': 7})

    if title!=None:
        fig.suptitle(title)
    if temp==True:
        secax=ax.secondary_xaxis('top',functions=(const.smear2temp,const.temp2smear))
        secax.set_xlabel('T [K]')
    if grid == True:
        ax.grid()
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()


def phonons_vs_Kgrid_cutoff(data,freqs=None,grid=True,save_as=None,axis=None,Kgrids=None,title=None):
    """It plots the frequencies as a function of Kgrid for different cutoffs
    
    data = Either the data, or folder where data is stored (it reads the foo.dyn1 files.)
    freqs = Select until which frequency you want to plot (integer).
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.
    title = A title of your liking

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data)
        Kgrids=data[0::2]
        data=conv.reverse_data(data)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
    else:
        ax=axis

    if freqs==None:
        freqs=len(data[1][0])
    freqs=freqs+1

    cmap=__get_cmap(int(len(data)/2))
    for i in range(0,len(data),2):
        color=cmap(int(i/2))
        plot_data=data[i+1]
        ax.plot(plot_data[:,0],plot_data[:,1],'.-',color=color,label=data[i])
        ax.plot(plot_data[:,0],plot_data[:,2:freqs],'.-',color=color)

    ax.set_ylabel("Freqs (cm-1)")
    ax.set_xlabel("K point number")
    grids_K=[]
    for g in Kgrids:
        n=np.prod([int(x) for x in g.split('x')])
        grids_K=grids_K+[n]
    ax.set_xticks(grids_K,labels=Kgrids,rotation=60)
    ax.legend(prop={'size': 7})
    
    if title!=None:
        fig.suptitle(title)
    if grid == True:
        ax.grid()
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()

def phonons_vs_Kgrid_smearing(data,freqs=None,grid=True,temp=False,save_as=None,axis=None,Kgrids=None,title=None):
    """It plots the frequencies as a function of Kgrid for different smearings
    
    data = Either the data, or folder where data is stored (it reads the foo.dyn1 files.)
    freqs = Select until which frequency you want to plot (integer).
    grid = Bolean that allows for a grid in the plot.
    temp = Bolean if you want a secondary axis with the Temperature equiv of the smearing value (usefull for Fermi-Dirac smearing).
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.
    title = A title of your liking

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the smearing number
    """
    if type(data)==str:
        data=read_data(data)
        Kgrids=data[0::2]
        data=conv.reverse_data(data)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
    else:
        ax=axis

    if freqs==None:
        freqs=len(data[1][0])
    freqs=freqs+1

    labels=data[0::2]
    if temp==True:
        for i,label in enumerate(labels):
            labels[i]=str(label)+" ("+str(int(const.smear2temp(label)))+"K)"

    cmap=__get_cmap(int(len(data)/2))
    for i in range(0,len(data),2):
        color=cmap(int(i/2))
        plot_data=data[i+1]
        ax.plot(plot_data[:,0],plot_data[:,1],'.-',color=color,label=labels[int(i/2)])
        ax.plot(plot_data[:,0],plot_data[:,2:freqs],'.-',color=color)

    ax.set_ylabel("Freqs (cm-1)")
    ax.set_xlabel("K point number")
    grids_K=[]
    for g in Kgrids:
        n=np.prod([int(x) for x in g.split('x')])
        grids_K=grids_K+[n]
    ax.set_xticks(grids_K,labels=Kgrids,rotation=60)
    ax.legend(prop={'size': 7})

    if title!=None:
        fig.suptitle(title)
    if grid == True:
        ax.grid()
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()


def phonons_cutoff(data,freqs=None,grid=True,save_as=None,title=None):
    """It plots the frequencies as a function of cutoffs and different k_grids
    
    data = Either the data, or folder where data is stored (it reads the foo.dyn1 files.)
    freqs = Select until which frequency you want to plot (integer).
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    title = A title of your liking

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data)
    data_K=conv.reverse_data(data)
    Kgrids=data[0::2]

    if freqs==None:
        freqs=len(data[1][0])
    
    w, h = plt.figaspect(0.5)    
    fig=plt.figure(figsize=(w, h))
    ax1 = fig.add_subplot(121)
    ax2 = fig.add_subplot(122)

    phonons_vs_cutoff(data,freqs=freqs,grid=grid,axis=ax1)
    phonons_vs_Kgrid_cutoff(data_K,freqs=freqs,grid=grid,axis=ax2,Kgrids=Kgrids)
    
    if title!=None:
        fig.suptitle(title)
    if save_as!=None:
        plt.savefig(save_as,dpi=300)
    plt.tight_layout()
    plt.show()

def phonons_Kgrid(data,freqs=None,grid=True,temp=False,save_as=None,title=None):
    """It plots the frequencies as a function of smearings and different k_grids
    
    data = Either the data, or folder where data is stored (it reads the foo.dyn1 files.)
    freqs = Select until which frequency you want to plot (integer).
    grid = Bolean that allows for a grid in the plot.
    temp = Bolean if you want a secondary axis with the Temperature equiv of the smearing value (usefull for Fermi-Dirac smearing).
    save_as = name.format in which to save your figure.
    title = A title of your liking

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the smearing number
    """
    if type(data)==str:
        data=read_data(data)
    data_K=conv.reverse_data(data)
    Kgrids=data[0::2]

    if freqs==None:
        freqs=len(data[1][0])
    
    w, h = plt.figaspect(0.5)    
    fig=plt.figure(figsize=(w, h))
    ax1 = fig.add_subplot(121)
    ax2 = fig.add_subplot(122)

    phonons_vs_smearing(data,freqs=freqs,grid=grid,temp=temp,axis=ax1)
    phonons_vs_Kgrid_smearing(data_K,freqs=freqs,grid=grid,temp=temp,axis=ax2,Kgrids=Kgrids)
    
    if title!=None:
        fig.suptitle(title)
    if save_as!=None:
        plt.savefig(save_as,dpi=300)
    plt.tight_layout()
    plt.show()



#PYTHON module for storing usefull constants:

Ry2eV = 13.6056980659
Ry2meV = Ry2eV*1000
Ry2jul= 2.179872e-18
eV2cm = 8065.544
Ry2cm = Ry2eV*eV2cm
jul2eV = Ry2eV/Ry2jul
Ry2cm_QE = 132.064879/1.20346372e-03 #fitted to match freqs
au2ang= 0.52917721067121
ang2metre=1e-10
hz2cm = 3.33565e-11  #checked
bohr2ang=au2ang
bohr2metre=bohr2ang*ang2metre
au2metre=bohr2metre
pas2bar=1e-5
Boltz=1.380649e-23
u2Kg = 1.66054e-27
me = 9.1093837e-31

def smear2temp(smear):
    temp=(Ry2jul*smear)/Boltz
    return temp

def temp2smear(temp):
    smear=(Boltz*temp)/Ry2jul
    return smear

def F_Ry2meV(e):
    return e*Ry2eV*1000

def F_meV2Ry(e):
    return e/(Ry2eV*1000)



#PYTHON module for cutoff convergence analysis

import re
import glob
import numpy as np
import matplotlib.pyplot as plt

import yaiv.constants as const
import yaiv.utils as utils

def __sort_Kgrids(Kgrids):
    """Using the list of Kgrids as input ['4x4x1','3x3x1','10x10x1']
    this returns a sorted list for the grids by the total number of K points"""
    num_k=[]
    for grid in Kgrids:
        x,y,z=[int(i) for i in grid.split('x')]
        num_k=num_k+[x*y*z]
    OUT=[Kgrids for _, Kgrids in sorted(zip(num_k, Kgrids))]
    return OUT

def __read_number_of_atoms(file):
    """Read the number of atoms from que Quantum Espresso output"""
    lines=open(file,'r')
    for line in lines:
        if re.search('number of atoms/cell',line):
            l=line.split()
            atoms_num=int(l[4])
    return atoms_num

def read_data(folder,shift=True):
    """from the folder where data is stored it reads the scf.pwo files (it autodetects the file extension .pwo, .out, whatever...)
    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number

    shift = Bolean regarding the option os shifting the total energies to zero.
    ...
    OUTPUT:
    A list where odd numbers are the Kdrid and even numbers are the corresponding folder info:
    [[   80.         -1416.5169454     -1.489      time          10GB    0.17]
     [   90.         -1416.51803251    -1.489      time          30Gb     0.22]
     [  cutoff(Ry)    energy(meV/atom)  fermi(eV)  time(hours)    RAM    forces(meV/au*atom)]
     [  110.         -1416.52358896    -1.4889     time    ...]
     [  120.         -1416.5240187     -1.4889     time    ...]]
     
    """
    data=[]
    Kgrids=glob.glob(folder+"/*")
    Kgrids=[i.split("/")[-1] for i in Kgrids]
    Kgrids=__sort_Kgrids(Kgrids)

    #Grep output extension
    subfolders=glob.glob(folder+"/"+Kgrids[0]+"/*")
    files=glob.glob(subfolders[0]+"/*")
    for i,file in enumerate(files):
        if utils.grep_filetype(file) == 'qe_scf_out':
            extension=file.split('.')[-1]
            break

    for i in range(len(Kgrids)):
        Kgrid=Kgrids[i]
        files=glob.glob(folder+"/"+Kgrid+"/*/*"+extension)
        grid_data=np.zeros([len(files),6])
        atoms_num=__read_number_of_atoms(files[0])
        for j in range(len(files)):
            cutoff=files[j].split("/")[-2]
            forces=0
            file = open (files[j],"r")
            for line in file:
                if re.search("!",line):
                    energy=line.split()[4]
                if  re.search("highest",line):
                    fermi=line.split()[4]
                if re.search("Fermi",line):
                    fermi=line.split()[4]
                if re.search("PWSCF.*WALL",line):
                    time=line.split('CPU')[1].split('WALL')[0]
                    h=0
                    m=0
                    s=0
                    if 'h' in time:
                        h=int(time.split('h')[0])
                        time=time.split('h')[1]
                    if 'm' in time:
                        m=int(time.split('m')[0])
                        time=time.split('m')[1]
                    if 's' in time:
                        s=float(time.split('s')[0])
                    time=(s+60*(m+60*h))/3600
                if re.search("total.*RAM",line):
                    if line.split()[6]=='MB':
                        RAM=str(float(line.split()[5])/1024)
                    else:
                        RAM=line.split()[5]
                if re.search("Total.*force",line):
                    forces=line.split()[3]
            grid_data[j,:]=np.array([cutoff,float(energy)*const.Ry2meV/atoms_num,fermi,time,RAM,float(forces)*const.Ry2meV/atoms_num])
        data=data+[Kgrid]+[grid_data]
    for i in range(0,len(data),2):
        data[i+1]=plot_data=data[i+1][data[i+1][:,0].argsort()] #sort acording to first column (x axis)(cutoff)

    if shift==True:
        MIN=None
        for d in data[1::2]:        #Get the minimum total energy
            m=np.min(d[:,1])
            if MIN==None:
                MIN=m
            elif MIN>m:
                MIN=m
        for i,d in enumerate(data[1::2]):          #Shift all the data accordingly
            data[2*i+1][:,1]=d[:,1]-MIN

    return data

def reverse_data(data):
    """From the data obtained with read_data it reverses so that the format is:
    ...
    OUTPUT:
    A list where odd numbers are the cutoffs and even numbers are the corresponding info:
    [[   K1xK2xK3         -1416.5169454     -1.489      time          10GB    0.17]
     [   K1xK2xK3         -1416.51803251    -1.489      time          30Gb     0.22]
     [  K1xK2xK3    energy(meV/atom)  fermi(eV)  time(hours)    RAM    forces(meV/au*atom)]
     [  K1xK2xK3         -1416.52358896    -1.4889     time    ...]
     [  K1xK2xK3         -1416.5240187     -1.4889     time    ...]]
     
    where first column is the total of K points, the secondo is the total energy(per atom in meV) and the third is the Fermi Energy...
    """

    Kgrids=data[0::2]
    cutoffs=[]
    for dataset in data[1::2]:
        for cutoff in dataset[:,0]:
            if cutoff not in cutoffs:
                cutoffs=cutoffs+[cutoff]
    new_data=[]
    for cutoff in cutoffs:
        for i,dataset in enumerate(data[1::2]):
            index=np.where(dataset[:,0]==cutoff)
            if len(index[0])==1:
                index=index[0][0]
                grid_K=np.prod([int(x) for x in Kgrids[i].split('x')])
                data_row=np.hstack((grid_K,dataset[index,1:]))
                try:
                    data_cutoff=np.vstack((data_cutoff,data_row))
                except NameError:
                    data_cutoff=data_row
            elif len(index)>1:
                print('Something really wrong happend...')
        new_data=new_data+[cutoff]+[data_cutoff]
        del data_cutoff
    return new_data

def energy_vs_cutoff(data,grid=True,save_as=None,axis=None,shift=True):
    """It plots the energy as a function of cutoff for different k_grids

    data = Either the data, or folder where data is stored it reads the scf.pwo files and plots.
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.
    shift = Bolean regarding the option os shifting the total energies to zero.

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data,shift=shift)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
    else:
        ax=axis

    for i in range(0,len(data),2):
        plot_data=data[i+1]
        ax.plot(plot_data[:,0],plot_data[:,1],'.-',label=data[i])
    ax.set_ylabel("total energy/atom (meV)")
    ax.set_xlabel("cutoff (Ry)")
    ax.legend(prop={'size': 7})
    if grid == True:
        ax.grid()
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()


def energy_vs_Kgrid(data,grid=True,save_as=None,axis=None,shift=True,Kgrids=None):
    """It plots the total energy as a function of K_grid for different cutoffs
    
    data = Either the data, or folder where data is stored it reads the scf.pwo files and plots.
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.
    shift = Bolean regarding the option os shifting the total energies to zero.

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data,shift=shift)
        Kgrids=data[0::2]
        data=reverse_data(data)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
    else:
        ax=axis

    for i in range(0,len(data),2):
        plot_data=data[i+1]
        ax.plot(plot_data[:,0],plot_data[:,1],'.-',label=int(data[i]))
    ax.set_ylabel("total energy/atom (meV)")
    ax.set_xlabel("K point number")
    grids_K=[]
    for g in Kgrids:
        n=np.prod([int(x) for x in g.split('x')])
        grids_K=grids_K+[n]
    ax.set_xticks(grids_K,labels=Kgrids,rotation=60)
    ax.legend(prop={'size': 7})
    if grid == True:
        ax.grid()
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()

def fermi_vs_cutoff(data,grid=True,save_as=None,axis=None):
    """It plots the Fermi energy as a function of cutoff for different k_grids
    
    data = Either the data, or folder where data is stored it reads the scf.pwo files and plots.
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
    else:
        ax=axis

    for i in range(0,len(data),2):
        plot_data=data[i+1]
        ax.plot(plot_data[:,0],plot_data[:,2],'.-',label=data[i])
    ax.set_ylabel("Fermi energy (eV)")
    ax.set_xlabel("cutoff (Ry)")
    ax.legend(prop={'size': 7})
    if grid == True:
        ax.grid()
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()


def fermi_vs_Kgrid(data,grid=True,save_as=None,axis=None,Kgrids=None):
    """It plots the Fermi energy as a function of K_grid for different cutoffs
    
    data = Either the data, or folder where data is stored it reads the scf.pwo files and plots.
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data)
        Kgrids=data[0::2]
        data=reverse_data(data)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
    else:
        ax=axis

    for i in range(0,len(data),2):
        plot_data=data[i+1]
        ax.plot(plot_data[:,0],plot_data[:,2],'.-',label=int(data[i]))
    ax.set_ylabel("Fermi energy (eV)")
    ax.set_xlabel("K point number")
    grids_K=[]
    for g in Kgrids:
        n=np.prod([int(x) for x in g.split('x')])
        grids_K=grids_K+[n]
    ax.set_xticks(grids_K,labels=Kgrids,rotation=60)
    ax.legend(prop={'size': 7})
    if grid == True:
        ax.grid()
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()

def time_vs_cutoff(data,grid=True,save_as=None,axis=None):
    """It plots the computation time as a function of cutoff for different k_grids
    
    data = Either the data, or folder where data is stored it reads the scf.pwo files and plots.
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
    else:
        ax=axis

    for i in range(0,len(data),2):
        plot_data=data[i+1]
        ax.plot(plot_data[:,0],plot_data[:,3],'.-',label=data[i])
    ax.set_ylabel("time in hours")
    ax.set_xlabel("cutoff (Ry)")
    ax.legend(prop={'size': 7})
    if grid == True:
        ax.grid()
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()


def time_vs_Kgrid(data,grid=True,save_as=None,axis=None,Kgrids=None):
    """It plots the computation time as a function of K_grid for different cutoffs
    
    data = Either the data, or folder where data is stored it reads the scf.pwo files and plots.
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data)
        Kgrids=data[0::2]
        data=reverse_data(data)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
    else:
        ax=axis

    for i in range(0,len(data),2):
        plot_data=data[i+1]
        ax.plot(plot_data[:,0],plot_data[:,3],'.-',label=int(data[i]))
    ax.set_ylabel("time in hours")
    ax.set_xlabel("K point number")
    grids_K=[]
    for g in Kgrids:
        n=np.prod([int(x) for x in g.split('x')])
        grids_K=grids_K+[n]
    ax.set_xticks(grids_K,labels=Kgrids,rotation=60)
    ax.legend(prop={'size': 7})
    if grid == True:
        ax.grid()
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()


def RAM_vs_cutoff(data,grid=True,save_as=None,axis=None):
    """It plots the RAM as a function of cutoff for different k_grids
    
    data = Either the data, or folder where data is stored it reads the scf.pwo files and plots.
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
    else:
        ax=axis

    for i in range(0,len(data),2):
        plot_data=data[i+1]
        ax.plot(plot_data[:,0],plot_data[:,4],'.-',label=data[i])
    ax.set_ylabel("RAM (Gb)")
    ax.set_xlabel("cutoff (Ry)")
    ax.legend(prop={'size': 7})
    if grid == True:
        ax.grid()
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()


def RAM_vs_Kgrid(data,grid=True,save_as=None,axis=None,Kgrids=None):
    """It plots the RAM as a function of K_grid for different cutoffs
    
    data = Either the data, or folder where data is stored it reads the scf.pwo files and plots.
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data)
        Kgrids=data[0::2]
        data=reverse_data(data)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
    else:
        ax=axis

    for i in range(0,len(data),2):
        plot_data=data[i+1]
        ax.plot(plot_data[:,0],plot_data[:,4],'.-',label=int(data[i]))
    ax.set_ylabel("RAM (Gb)")
    ax.set_xlabel("K point number")
    grids_K=[]
    for g in Kgrids:
        n=np.prod([int(x) for x in g.split('x')])
        grids_K=grids_K+[n]
    ax.set_xticks(grids_K,labels=Kgrids,rotation=60)
    ax.legend(prop={'size': 7})
    if grid == True:
        ax.grid()
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()

def forces_vs_cutoff(data,grid=True,save_as=None,axis=None):
    """It plots the (total force)/(num atoms) as a function of cutoff for different k_grids
    
    data = Either the data, or folder where data is stored it reads the scf.pwo files and plots.
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
    else:
        ax=axis

    for i in range(0,len(data),2):
        plot_data=data[i+1]
        ax.plot(plot_data[:,0],plot_data[:,5],'.-',label=data[i])
    ax.set_ylabel("Total force/atom (meV/au)")
    ax.set_xlabel("cutoff (Ry)")
    ax.legend(prop={'size': 7})
    if grid == True:
        ax.grid()
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()


def forces_vs_Kgrid(data,grid=True,save_as=None,axis=None,Kgrids=None):
    """It plots the num atomsRAM as a function of K_grid for different cutoffs
    
    data = Either the data, or folder where data is stored it reads the scf.pwo files and plots.
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data)
        Kgrids=data[0::2]
        data=reverse_data(data)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
    else:
        ax=axis

    for i in range(0,len(data),2):
        plot_data=data[i+1]
        ax.plot(plot_data[:,0],plot_data[:,5],'.-',label=int(data[i]))
    ax.set_ylabel("Total force/atom (meV/au)")
    ax.set_xlabel("K point number")
    grids_K=[]
    for g in Kgrids:
        n=np.prod([int(x) for x in g.split('x')])
        grids_K=grids_K+[n]
    ax.set_xticks(grids_K,labels=Kgrids,rotation=60)
    ax.legend(prop={'size': 7})
    if grid == True:
        ax.grid()
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()


def energy(data,grid=True,save_as=None,shift=True):
    """It plots the total energy as a function of cutoffs and grids
    
    data = Either the data, or folder where data is stored it reads the scf.pwo files and plots.
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    shift = Bolean regarding the option os shifting the total energies to zero.

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data,shift=shift)
    data_K=reverse_data(data)
    Kgrids=data[0::2]
    
    w, h = plt.figaspect(0.5)    
    fig=plt.figure(figsize=(w, h))
    fig.suptitle('Total energy')
    ax1 = fig.add_subplot(121)
    ax2 = fig.add_subplot(122)

    energy_vs_cutoff(data,grid=grid,axis=ax1)
    energy_vs_Kgrid(data_K,grid=grid,axis=ax2,Kgrids=Kgrids)
    
    if save_as!=None:
        plt.savefig(save_as,dpi=300)
    plt.tight_layout()
    plt.show()


def fermi(data,grid=True,save_as=None):
    """It plots the total fermi energy as a function of cutoffs and grids
    
    data = Either the data, or folder where data is stored it reads the scf.pwo files and plots.
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data)
    data_K=reverse_data(data)
    Kgrids=data[0::2]
    
    w, h = plt.figaspect(0.5)    
    fig=plt.figure(figsize=(w, h))
    fig.suptitle('Fermi Energy')
    ax1 = fig.add_subplot(121)
    ax2 = fig.add_subplot(122)

    fermi_vs_cutoff(data,grid=grid,axis=ax1)
    fermi_vs_Kgrid(data_K,grid=grid,axis=ax2,Kgrids=Kgrids)
    
    if save_as!=None:
        plt.savefig(save_as,dpi=300)
    plt.tight_layout()
    plt.show()


def time(data,grid=True,save_as=None):
    """It plots the total computation time as a function of cutoffs and grids
    
    data = Either the data, or folder where data is stored it reads the scf.pwo files and plots.
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data)
    data_K=reverse_data(data)
    Kgrids=data[0::2]
    
    w, h = plt.figaspect(0.5)    
    fig=plt.figure(figsize=(w, h))
    fig.suptitle('Computing time')
    ax1 = fig.add_subplot(121)
    ax2 = fig.add_subplot(122)

    time_vs_cutoff(data,grid=grid,axis=ax1)
    time_vs_Kgrid(data_K,grid=grid,axis=ax2,Kgrids=Kgrids)
    
    if save_as!=None:
        plt.savefig(save_as,dpi=300)
    plt.tight_layout()
    plt.show()

def RAM(data,grid=True,save_as=None):
    """It plots the total needed RAM as a function of cutoffs and grids
    
    data = Either the data, or folder where data is stored it reads the scf.pwo files and plots.
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data)
    data_K=reverse_data(data)
    Kgrids=data[0::2]
    
    w, h = plt.figaspect(0.5)    
    fig=plt.figure(figsize=(w, h))
    fig.suptitle('RAM (GB)')
    ax1 = fig.add_subplot(121)
    ax2 = fig.add_subplot(122)

    RAM_vs_cutoff(data,grid=grid,axis=ax1)
    RAM_vs_Kgrid(data_K,grid=grid,axis=ax2,Kgrids=Kgrids)
    
    if save_as!=None:
        plt.savefig(save_as,dpi=300)
    plt.tight_layout()
    plt.show()

def forces(data,grid=True,save_as=None):
    """It plots the total (total force)/(num atoms) as a function of cutoffs and grids
    
    data = Either the data, or folder where data is stored it reads the scf.pwo files and plots.
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data)
    data_K=reverse_data(data)
    Kgrids=data[0::2]
    
    w, h = plt.figaspect(0.5)    
    fig=plt.figure(figsize=(w, h))
    fig.suptitle('Total force')
    ax1 = fig.add_subplot(121)
    ax2 = fig.add_subplot(122)

    forces_vs_cutoff(data,grid=grid,axis=ax1)
    forces_vs_Kgrid(data_K,grid=grid,axis=ax2,Kgrids=Kgrids)
    
    if save_as!=None:
        plt.savefig(save_as,dpi=300)
    plt.tight_layout()
    plt.show()


def analysis(data,grid=True,save_as=None,shift=True,title=None):
    """It plots all the posible comparisons for a full convergence analysis 
    
    data = Either the data, or folder where data is stored it reads the scf.pwo files and plots.
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    shift = Bolean regarding the option os shifting the total energies to zero.
    title = A title of your liking

    Data must be organized with parent folders with the K grid as:
    K1xK2xK3
    And subfolders with the cutoff number
    """
    if type(data)==str:
        data=read_data(data,shift=shift)
    data_K=reverse_data(data)
    Kgrids=data[0::2]

    fig,ax=plt.subplots(5,2,figsize=(10.5,14.5))

    energy_vs_cutoff(data,grid=grid,axis=ax[0,0])
    energy_vs_Kgrid(data_K,grid=grid,axis=ax[0,1],Kgrids=Kgrids)

    forces_vs_cutoff(data,grid=grid,axis=ax[1,0])
    forces_vs_Kgrid(data_K,grid=grid,axis=ax[1,1],Kgrids=Kgrids)

    fermi_vs_cutoff(data,grid=grid,axis=ax[2,0])
    fermi_vs_Kgrid(data_K,grid=grid,axis=ax[2,1],Kgrids=Kgrids)

    time_vs_cutoff(data,grid=grid,axis=ax[3,0])
    time_vs_Kgrid(data_K,grid=grid,axis=ax[3,1],Kgrids=Kgrids)

    RAM_vs_cutoff(data,grid=grid,axis=ax[4,0])
    RAM_vs_Kgrid(data_K,grid=grid,axis=ax[4,1],Kgrids=Kgrids)
    
    if title!=None:
        fig.suptitle(title,y=0.99,size=16)

    plt.tight_layout()
    if save_as!=None:
        plt.savefig(save_as,dpi=300)
    plt.show()



#PYTHON module for cutoff convergence analysis

import re
import numpy as np
import matplotlib.pyplot as plt

# WANNIER90****************************************************************************************

def read_w90out(file):
    """reads w90out file in order to analyze the minimization
    returns:
    total_spread, final_spread_delta, wan_spreads, wan_centers, disentang, final_dis_delta
    """
    lines=open(file,'r')
    read_dis=False
    read_wan=False
    read_step=False
    iteration=-1
    disentang=0
    dis_delta=0
    for i,line in enumerate(lines):

        if line == '\n':
            if read_dis==True:
                dis_delta=float(l[3])
            read_dis=False
            read_step=False

        #READ disentanglement
        if read_dis==True:
            l=line.split()
            if l[0]=='1':
                i=int(l[0])
                O_i=float(l[1])
                disentang=np.array([i,O_i])
                read_step=True
            elif read_step==True:
                i=int(l[0])
                O_i=float(l[1])
                disentang=np.vstack([disentang,np.array([i,O_i])])

        #READ wannierization
        if read_wan==True:
            l=line.split()

            #Counts in which step you are
            if len(l)>5 and l[4]=='1':
                if iteration==0:
                    total_spread=new_t_spread
                    deltas=new_delta
                    wan_centers=new_centers
                    wan_spreads=new_spreads
                    wan_num=len(new_spreads)
                elif iteration>0:
                    total_spread=np.vstack([total_spread,new_t_spread])
                    deltas=np.vstack([deltas,new_delta])
                    if len(new_spreads) == wan_num:     #Check that the step is complete
                        wan_centers=np.vstack([wan_centers,new_centers])
                        wan_spreads=np.vstack([wan_spreads,new_spreads])
                    else:
                        iteration=iteration-1
                iteration=iteration+1
                new_centers=iteration
                new_spreads=iteration
            #Reads individual WF
            if re.search('WF centre and spread',line):
                spread=float(l[-1])
                if l[-5][0]=='(':
                    l[-5]=l[-5][1:]
                centre=np.array([float(l[-5][:-1]),float(l[-4][:-1]),float(l[-3])])
                new_spreads=np.hstack([new_spreads,spread])
                new_centers=np.hstack([new_centers,centre])
            if re.search('<-- CONV',line):
                new_t_spread=np.array([iteration,float(l[3])])
                new_delta=np.array([iteration,float(l[1])])

        if re.search('Extraction of optimally',line):
            read_dis=True
        if re.search('Initial State',line):
            read_wan=True
            Initial=True
        else:
            Initial=False
        if re.search('WANNIER90',line):
            read_wan=False

        if re.search('Number of Wannier',line):
            l=line.split()
            num_wan=int(l[6])

    spread_delta=deltas[-1,1]

    return total_spread,spread_delta,wan_spreads,wan_centers,disentang,dis_delta

def w90_wannierization(data,grid=True,save_as=None,axis=None):
    """ Plots the evolution of the wannierization of the w90 routine (total spread).

    data = Either the .wout file or the output of read_w90out(file).
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.
    """
    if type(data)==str:
        data=read_w90out(data)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
        textsize=12
    else:
        ax=axis
        textsize=10

    total_spread,spread_delta,wan_spreads,wan_centers,disentang,dis_delta=data

    data=total_spread
    delta=np.hstack([0,np.diff(data[:,1])])
    data=np.insert(data,2,delta,axis=1)
    ax.plot(data[:,0],data[:,1])
    #plt.errorbar(data[:,0],data[:,1],yerr=data[:,2],elinewidth=0.4)
    
    ax.text(data[-1,0]*0.8,data[0,1]-(data[0,1]-data[-1,1])*0.07
             , 'Final $\Delta\Omega$ ='+str(spread_delta)
             , size=textsize, ha="center", va="center",
             bbox=dict(boxstyle="round",
                       ec=(1., 0.5, 0.5),
                       fc=(1., 0.8, 0.8),
                       )
             )
    ax.set_title('Wannerization')
    ax.set_ylabel('$\Omega \ (\AA^2)$')
    ax.set_xlabel('Iteration')

    if grid == True:
        ax.grid(axis='both',color='0.95')   
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()

def w90_disentanglement(data,grid=True,save_as=None,axis=None):
    """ Plots the disentanglement procedure of the w90 routine (the gauge invariant spread).
    
    data = Either the .wout file or the output of read_w90out(file).
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.
    """
    if type(data)==str:
        data=read_w90out(data)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
        textsize=12
    else:
        ax=axis
        textsize=10

    total_spread,spread_delta,wan_spreads,wan_centers,disentang,dis_delta=data

    data=disentang
    ax.plot(data[:,0],data[:,1])

    ax.text(data[-1,0]*0.8,data[0,1]-(data[0,1]-data[-1,1])*0.07
             , 'Final $\Delta\Omega_I$ ='+str(dis_delta)
             , size=textsize, ha="center", va="center",
             bbox=dict(boxstyle="round",
                       ec=(1., 0.5, 0.5),
                       fc=(1., 0.8, 0.8),
                       )
             )
    
    ax.set_title('Disentanglement')
    ax.set_ylabel('$\Omega_I \ (\AA^2)$')
    ax.set_xlabel('Iteration')

    if grid == True:
        ax.grid(axis='both',color='0.95')   
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()

def w90_centers(data,grid=True,save_as=None,axis=None):
    """ Plots the evolution of the wannier centers of the w90 routine.
    
    data = Either the .wout file or the output of read_w90out(file).
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.
    """
    if type(data)==str:
        data=read_w90out(data)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
    else:
        ax=axis

    total_spread,spread_delta,wan_spreads,wan_centers,disentang,dis_delta=data
    
    data=wan_centers
    center_num=len(wan_centers[0,:])-1
    wan_num=len(wan_spreads[0,:])-1

    ax.plot(data[:,0],data[:,1],color='C0',linewidth=0.7,label='x')
    ax.plot(data[:,0],data[:,2],color='C1',linewidth=0.7,label='y')
    ax.plot(data[:,0],data[:,3],color='C2',linewidth=0.7,label='z')


    for n in range(4,center_num+1,3):
        ax.plot(data[:,0],data[:,n],color='C0',linewidth=0.7)
    for n in range(5,center_num+1,3):
        ax.plot(data[:,0],data[:,n],color='C1',linewidth=0.7)
    for n in range(6,center_num+1,3):
        ax.plot(data[:,0],data[:,n],color='C2',linewidth=0.7)        

    if axis==None:
        ax.set_title('WFs centers ('+str(wan_num)+')')
    else:
        ax.set_title('WFs centers')
    ax.set_ylabel('Center coord. ($\AA$)')
    ax.set_xlabel('Iteration')
    ax.legend()

    if grid == True:
        ax.grid(axis='both',color='0.95')   
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()

def w90_spreads(data,grid=True,save_as=None,axis=None):
    """ Plots the evolution of the individual spreads of the w90 routine.

    data = Either the .wout file or the output of read_w90out(file).
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.
    axis = Matplotlib axis in which to plot.
    """
    if type(data)==str:
        data=read_w90out(data)

    if axis == None:
        fig=plt.figure()
        ax = fig.add_subplot(111)
    else:
        ax=axis

    total_spread,spread_delta,wan_spreads,wan_centers,disentang,dis_delta=data

    data=wan_spreads
    wan_num=len(wan_spreads[0,:])-1
    for n in range(1,wan_num+1):
        ax.plot(data[:,0],data[:,n],color='C0',linewidth=0.7)
    
    if axis==None:
        ax.set_title('WFs spreads ('+str(wan_num)+')')
    else:
        ax.set_title('WFs spreads')
    ax.set_ylabel('$\Omega \ (\AA^2)$')
    ax.set_xlabel('Iteration')

    if grid == True:
        ax.grid(axis='both',color='0.95')   
    if save_as!=None:
        plt.tight_layout()
        plt.savefig(save_as,dpi=300)
        plt.show()
    if axis == None:
        plt.tight_layout()
        plt.show()


def w90(data,title=None,grid=True,save_as=None):
    """ Plots the evolution of Wannierization routine.
   
    data = Either the .wout file or the output of read_w90out(file).
    grid = Bolean that allows for a grid in the plot.
    save_as = name.format in which to save your figure.

    returns a figure where:

    ax1: Wannierization of the w90 routine (total spread)
    ax2: Disentanglement procedure of the w90 routine (the gauge invariant spread)
    ax3: Evolution of the individual spreads of the w90 routine
    ax4: Evolution of the wannier centers of the w90 routine
    """
    if type(data)==str:
        data=read_w90out(data)

    total_spread,spread_delta,wan_spreads,wan_centers,disentang,dis_delta=data
    wan_num=len(wan_spreads[0,:])-1

    fig = plt.figure(figsize=(9.0, 7.0)) #size of our plots (in inches)
    ax1=fig.add_subplot(2,2,1) #rows of sublots, colums of subplots, place of this figure
    ax2=fig.add_subplot(2,2,2)
    ax3=fig.add_subplot(2,2,3)
    ax4=fig.add_subplot(2,2,4)

    w90_wannierization(data,grid=grid,axis=ax1)
    w90_disentanglement(data,grid=grid,axis=ax2)
    w90_spreads(data,grid=grid,axis=ax3)
    w90_centers(data,grid=grid,axis=ax4)

    if title!=None:
        fig.suptitle(title,y=0.99,size=16)
    else:
        fig.suptitle('Wannierization ('+str(wan_num)+' WF)', fontsize=16)
    plt.tight_layout()
    if save_as!=None:
        plt.savefig(save_as,dpi=300)
    plt.show()



import yaiv.convergence.cutoff
import yaiv.convergence.kgrid
import yaiv.convergence.phonons
import yaiv.convergence.wannier
#import yaiv.experimental.convergence.smearing
#import yaiv.experimental.convergence.phonons
